/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endour or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.seasar.kijimuna.core.dicon.tools;

import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.seasar.kijimuna.core.Kijimuna;
import org.seasar.kijimuna.core.dicon.info.IPointcut;
import org.seasar.kijimuna.core.dicon.model.AspectElement;
import org.seasar.kijimuna.core.rtti.IRtti;
import org.seasar.kijimuna.core.rtti.IRttiMethodDesctiptor;
import org.seasar.kijimuna.core.util.ModelUtils;

/**
 * @author Masataka Kurihara (Gluegent, Inc.)
 */
public class Pointcut implements IPointcut {
    
    private AspectElement aspect;
    private String regexp;
    private String message;
    private Pattern pattern;
    private transient IRttiMethodDesctiptor[] methods;
    private boolean autoApply;
    
    public Pointcut(AspectElement aspect, String regexp) {
        this.aspect = aspect;
        if(regexp != null) {
	        this.regexp = regexp;
        } else {
            autoApply = true;
        }
        getPattern();
    }
    
    private Pattern getPattern() {
        if(pattern == null) {
	        try {
	            String r = regexp;
	            if(autoApply) {
	                r = ".*";
	            }
	            pattern = Pattern.compile(r);
	        } catch (PatternSyntaxException e) {
	            message = e.getMessage();
	        }
        }
        return pattern;
    }
    
    public IRttiMethodDesctiptor[] getApplyMethods() {
        if(methods == null) {
            methods = new IRttiMethodDesctiptor[0];
            if(autoApply) {
	            IRtti rtti = ModelUtils.getComponentRtti(aspect);
                if(rtti != null) {
                    IRtti[] interfaces = rtti.getInterfaces();
    	            Pattern p = getPattern();
    	            Set set = new TreeSet();
                    for(int i = 0; i < interfaces.length; i++) {
                        IRttiMethodDesctiptor[] desc = interfaces[i].getMethods(p);
                        for(int k = 0; k < desc.length; k++) {
                            IRttiMethodDesctiptor method = rtti.getMethod(
                                    desc[k].getMethodName(), desc[k].getArgs(), false);
                            set.add(method);
                        }
                    }
                    methods = (IRttiMethodDesctiptor[])set.toArray(
                            new IRttiMethodDesctiptor[set.size()]);
                }
            } else {
	            Pattern p = getPattern();
	            if(p != null) {
	            	IRtti rtti = ModelUtils.getComponentRtti(aspect);
	                if(rtti != null) {
	                	methods = rtti.getMethods(p);
	                }
	            }
            }
        }
        return methods;
    }

    public String getRegexp() {
        if(regexp != null) {
            return regexp;
        } else {
            return Kijimuna.getResourceString("dicon.tools.Pointcut.1");
        }
    }

    public int compareTo(Object test) {
        if(test instanceof IPointcut) {
            IPointcut pointcut = (IPointcut)test;
            return getRegexp().compareTo(pointcut.getRegexp());
        } else {
            return 0;
        }
    }
    
    public String getErrorMessage() {
        return message;
    }

    public boolean hasError() {
        return (message != null);
    }

}
