/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endour or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.seasar.kijimuna.core.dicon.validation;

import org.seasar.kijimuna.core.ConstCore;
import org.seasar.kijimuna.core.dicon.IValidation;
import org.seasar.kijimuna.core.dicon.info.IComponentNotFound;
import org.seasar.kijimuna.core.dicon.info.IApplyMethodInfo;
import org.seasar.kijimuna.core.dicon.info.ITooManyRegisted;
import org.seasar.kijimuna.core.dicon.model.MethodElement;
import org.seasar.kijimuna.core.parser.Element;
import org.seasar.kijimuna.core.rtti.IRtti;
import org.seasar.kijimuna.core.rtti.IRttiMethodDesctiptor;
import org.seasar.kijimuna.core.util.ModelUtils;
import org.seasar.kijimuna.core.util.StringUtils;

/**
 * @author Masataka Kurihara (Gluegent, Inc.)
 */
public class AutoMethodInvoke implements IValidation, ConstCore {
	
    public void validation(Element element) {
        if(element instanceof MethodElement) {
            try {
            	invoke((MethodElement)element);
            } catch(Exception e) {
            	e.printStackTrace();
            }
        }
    }

	private void invoke(MethodElement method) {
		String methodName = method.getMethodName();
		if((StringUtils.noneValue(methodName)) ||
		        (method.getArgList().size() > 0)) {
			return;
		}
		IRtti component = ModelUtils.getComponentRtti(method);
		IApplyMethodInfo info = (IApplyMethodInfo)method.getAdapter(IApplyMethodInfo.class);
		if(info != null) {
		    IRttiMethodDesctiptor suitable = info.getAutoInjectedMethod();
		    if(suitable != null) {
			    IRtti[] suitableArgs = suitable.getArgs();
			    IRtti[] injectedArgs = suitable.getValues();
				String display = ModelUtils.getMethodDisplay(suitable, true);
		        if(injectedArgs != null) {
		            for(int i = 0; i < suitableArgs.length; i++) {
		                if(injectedArgs[i] instanceof IComponentNotFound) {
							MarkerSetting.createDiconMarker(
							        "dicon.validation.AutoMethodInvoke.2",
							        method, new Object[] { display, new Integer(i + 1) });
		                } else if(injectedArgs[i] instanceof ITooManyRegisted) {
							MarkerSetting.createDiconMarker(
							        "dicon.validation.AutoMethodInvoke.4",
							        method, new Object[] { display, new Integer(i + 1) });
		                } else {
							MarkerSetting.createDiconMarker(
		                        "dicon.validation.AutoMethodInvoke.1",
		                        method, new Object[] {
		                        	display, new Integer(i + 1),
		                        	ModelUtils.getInjectedElementName(injectedArgs[i]) });
		                }
		            }
		        }
		    } else {
				String display = ModelUtils.getMethodDisplay(
						component, methodName, new IRtti[0], true);
				MarkerSetting.createDiconMarker(
				        "dicon.validation.AutoMethodInvoke.3",
				        method, new Object[] { display });
		    }
		}
	}
}
