/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endour or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.seasar.kijimuna.ui.dicon.provider.property;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.seasar.kijimuna.core.rtti.IRtti;
import org.seasar.kijimuna.core.rtti.IRttiMethodDesctiptor;
import org.seasar.kijimuna.ui.KijimunaUI;

/**
 * @author Masataka Kurihara (Gluegent, Inc.)
 */
public class MethodProperty extends NullProperty {

    private static IPropertyDescriptor[] descriptors;
    
    static {
        // info category
        String category = KijimunaUI.getResourceString(
                "dicon.provider.property.ContentProperty.4");
        descriptors = new IPropertyDescriptor[6];
        // args
        String id = "dicon.provider.property.MethodProperty.1";
        PropertyDescriptor d = 
            new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[0] = d;
        // returnType
        id = "dicon.provider.property.MethodProperty.2";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[1] = d;
        // methodName
        id = "dicon.provider.property.MethodProperty.3";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[2] = d;
        // final
        id = "dicon.provider.property.MethodProperty.4";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[3] = d;
        // static
        id = "dicon.provider.property.MethodProperty.5";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[4] = d;
        // parent
        id = "dicon.provider.property.MethodProperty.6";
        d = new PropertyDescriptor(id, KijimunaUI.getResourceString(id));
        d.setCategory(category);
        descriptors[5] = d;
    }

    private IRttiMethodDesctiptor method;
    
    public MethodProperty(IRttiMethodDesctiptor method) {
    	this.method = method;
    }
    
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return descriptors;
	}

	private String getArgDisplay() {
		IRtti[] args = method.getArgs();
		StringBuffer ret = new StringBuffer();
		for(int i = 0; i < args.length; i++) {
			if (i != 0) {
				ret.append(", ");
			}
			ret.append(args[i].getQualifiedName());
		}
		return ret.toString();
	}
	
	public Object getPropertyValue(Object id) {
		if("dicon.provider.property.MethodProperty.1".equals(id)) {
			return getArgDisplay();
		} else if ("dicon.provider.property.MethodProperty.2".equals(id)) {
			return method.getReturnType();
		} else if ("dicon.provider.property.MethodProperty.3".equals(id)) {
			return method.getMethodName();
		} else if ("dicon.provider.property.MethodProperty.4".equals(id)) {
			return new Boolean(method.isFinal());
		} else if ("dicon.provider.property.MethodProperty.5".equals(id)) {
			return new Boolean(method.isStatic());
		} else if ("dicon.provider.property.MethodProperty.6".equals(id)) {
			return method.getParent();
		} else {
			return null;
		}
	}
	
	
}
